﻿Type=Service
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'-------------------------------------------------------------------------------------------------------
'                                        MQTT-Service
'                                        ------------
'																	   Tom Broeger 14.11.2015
' With reconnect if connection breaks.
'-------------------------------------------------------------------------------------------------------

#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Dim mqtt As MqttClient
	Dim Notification1 As Notification
	Dim tiTryRec As Timer 'Reconnect Timer
	Dim recMs As Long=10000 'reconnect cycle as ms !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Dim connectFirstTry As Boolean 'connect first try
End Sub

Sub Service_Create
	'---Notification init
	Notification1.Initialize
	Notification1.Icon="icon" 'use the application icon file for the notification
	Notification1.Vibrate=False
	Notification1.Light=True
	Notification1.Sound=False
	Notification1.AutoCancel=True
End Sub

Sub Service_Start (StartingIntent As Intent)
	'---process stays alive
	Notification1.SetInfo("MQTT Service started", "", Main)
	Service.StartForeground(1, Notification1) 
	ToastMessageShow("MQTT Service started",False)
	'---MQTT-Broker Connect
	tiTryRec.Initialize("tiTryRec",recMs) 'init reconnect timer
	tiTryRec.Enabled=False
	connectFirstTry=True
	connectMQTT
End Sub

Sub Service_Destroy
'	Service.StopForeground(1) 'not necessary
	tiTryRec.Enabled=False
	If mqtt.Connected Then  mqtt.Close 'Event mqtt_Disconnected triggers, but discarded with Service_Destroy
End Sub



'####################################### MQTT-Events ######################################################

Sub mqtt_Connected(Success As Boolean)
'----------------------------------------------------------
'	Event triggers after connection
'----------------------------------------------------------
	Notification1.Sound=False
	Notification1.Light=False
	If Success=False Then
		'---noti only 1. time
		If connectFirstTry Then
			connectFirstTry=False
			Log(LastException)
			Dim x As String=recMs/1000
			Notification1.SetInfo("MQTT Connect-Error","Try Reconnect every "&x&"s", Main) 
			Notification1.Notify(1)
		End If
		'---try again in xx ms
		tiTryRec.Enabled=True
	Else
		'---do Subscribs
		mqtt.Subscribe("input/",mqtt.QOS_0_MOST_ONCE)
		mqtt.Subscribe("status",mqtt.QOS_0_MOST_ONCE)
		'---Publish Topic status
'		mqtt.Publish2("status",strToAsciiBytes("Online"),0,True)
		'---Noti
		Notification1.SetInfo("MQTT Connected", "", Main)
		Notification1.Notify(1)
	End If
End Sub

Sub mqtt_Disconnected
'---------------------------------------------------------
'	triggers if connection breaks
'---------------------------------------------------------
	'---Noti
	Notification1.Sound=True
	Notification1.Light=True
	Dim x As String=recMs/1000
	Notification1.SetInfo("MQTT Disconnected","Try Reconnect in max. "&x&"s", Main)
	Notification1.Notify(1)
	
	'---try reconnect in xx ms
	connectFirstTry=True
	tiTryRec.Enabled=True
End Sub

Sub tiTryRec_Tick
'---try Reconnect Timer
	Log("--------------->rec-tick")
	tiTryRec.Enabled=False
	connectMQTT 'try again
End Sub

Sub mqtt_MessageArrived(Topic As String, Payload() As Byte)
'-------------------------------------------------------------------
'	triggers if new message arrives
'-------------------------------------------------------------------
	'---Bytes to string
	Dim msg As String=BytesToString(Payload,0,Payload.Length,"UTF-8")
	msg=DateTime.Date(DateTime.Now)&" "&DateTime.Time(DateTime.Now)&"  "&Topic &"<>"& msg
	'---add to list
	Main.messages.Add(msg)
	'---show or Noti
	If IsPaused(Main)=False Then 'Activity is foreground
		CallSub(Main,"zeigeMessages")
		ToastMessageShow("New Message arrived",True)
	Else
		Main.unreadMess=Main.unreadMess+1 'unread messages
		'---noti only at first message
		If Main.unreadMess=1 Then
'			Notification1.Number=Main.unreadMess
			Notification1.Light=True
			Notification1.Sound = True
			Notification1.SetInfo("MQTT Connected","Unread Message/s", Main)
			Notification1.Notify(1)
		End If
	End If
End Sub


'###################################### Subs ##############################################################

Sub connectMQTT
'---------------------------------------
'	connect with broker
'---------------------------------------
	mqtt.Initialize("mqtt", Main.serverURI, Rnd(0, 999999999) & DateTime.Now) 'EventID,Server,ClientID
	'---Connect-Options
	Dim mo As MqttConnectOptions
	mo.Initialize(Main.user,Main.password) 'User,Password
	mo.CleanSession=True
	Dim lw As String="Just crashed!" 'Last Will
	mo.SetLastWill("status",strToAsciiBytes(lw),mqtt.QOS_0_MOST_ONCE,False)
	'---connect
	mqtt.Connect2(mo) 'triggers Event 'mqtt_Connected'
End Sub

Sub publish
'---Publish Topic
	Dim txt As String=DateTime.Time(DateTime.Now)
	If mqtt.Connected Then
		mqtt.Publish2("input/",strToAsciiBytes(txt),0,False)
	Else
		ToastMessageShow("MQTT not Connected!",False)
	End If 
End Sub

Sub clrNoti
'---clear Noti new messages
	Notification1.Number=0
	Notification1.Sound=False
	Notification1.SetInfo("MQTT Connected","",Main) 'Message gelesen
	Notification1.Notify(1)
End Sub


Sub strToAsciiBytes(s As String) As Byte()
'---cast String to ASCII-Array
	Return s.GetBytes("UTF-8")
End Sub
